	LIST P=12F675, R=DEC
	include P12F675.INC
	LIST

	__CONFIG	_CPD_OFF &  _CP_OFF & _BODEN_ON & _MCLRE_OFF & _PWRTE_ON & _WDT_ON & _INTRC_OSC_NOCLKOUT

;==================================================================
;       1 .   
; 64 uS (15,625 kHz).    7  6  TMR0    .
;           OUT_HIGH  OUT_LOW.
;
;      ,     
;  .         .   
;    OUT_HIGH  OUT_LOW  .
;******************************************************************
; Definitions
;==================================================================
;
; I/O, Interrupt and Option Definitions
;
	constant	C_OPTION_VAL = B'11011000'	
				;   			
				;       INT (   )
				;    TMR0
				; TMR0      (   )
				;   WDT
				;  1/1

	constant	C_INTCONVAL = B'10100000'	; set GIE, T0IE

	constant	C_TRIS_VAL   = B'11011100'	; GP0, GP1, GP5 output
	constant	C_AN_SEL_VAL = B'01100000'	; ADC  1/64 Tosc,   

	constant	C_ADCON0VAL = B'00000000' ;   
;--------------------------------------------------------------------
	constant	C_GLOBAL_GONG_TIME = D'152'   ;      . 10 sec.

	constant	C_TONE_1_MAX_VOLUME = D'20'	;  ()   .
	constant	C_TONE_2_MAX_VOLUME = D'24'
	constant	C_TONE_3_MAX_VOLUME = D'30'

	constant	C_TONE_1_BEGIN_TIME = C_GLOBAL_GONG_TIME - D'00'	;     
	constant	C_TONE_2_BEGIN_TIME = C_GLOBAL_GONG_TIME - D'18'	;  65536  uS.
	constant	C_TONE_3_BEGIN_TIME = C_GLOBAL_GONG_TIME - D'36'

	constant	C_TONE_1_LENGHT = D'67'	; 67 * 65536 uS = 4,4 Sec
	constant	C_TONE_2_LENGHT = D'67'	; 67 * 65536 uS = 4,4 Sec
	constant	C_TONE_3_LENGHT = D'67'	; 67 * 65536 uS = 4,4 Sec

	constant	C_TONE_1_DECREMENT_STEP = C_TONE_1_LENGHT / C_TONE_1_MAX_VOLUME
	constant	C_TONE_2_DECREMENT_STEP = C_TONE_2_LENGHT / C_TONE_2_MAX_VOLUME
	constant	C_TONE_3_DECREMENT_STEP = C_TONE_3_LENGHT / C_TONE_2_MAX_VOLUME

;---------------
	constant	C_TONE_1_PHASE_START = D'10'	;     .
	constant	C_TONE_2_PHASE_START = D'12'	;   .
	constant	C_TONE_3_PHASE_START = D'15'
;==================================================================
	#define		DING_DANG_IN		GPIO,4
	#define		DING_DANG_DONG_IN	GPIO,2

	#define		TONE_PIN	GPIO,5	; GPIO,0	GPIO,1
;==================================================================
; Register Definitions

	CBLOCK	H'20'

	W_TEMP		; stack to push/pop the W-register
	STATUS_TEMP

	OUT_HIGH	;   
	OUT_LOW

	GLOBAL_GONG_TMR		;      .  65536 uS.

	TONE_1_VOLUME	;    
	TONE_2_VOLUME
	TONE_3_VOLUME

	COUNTER_TONE_1_DOWN		;      
	COUNTER_TONE_2_DOWN		;  
	COUNTER_TONE_3_DOWN

	PHASE_TONE_1_COUNTER		;      .
	PHASE_TONE_2_COUNTER		;  '00'    
	PHASE_TONE_3_COUNTER




	ENDC

;******* RESET ****************************************************
	ORG H'00'
	goto START

;*******  ISR  ****************************************************
	ORG H'04'		; timer interrupt

	btfsc	TMR0,0		; compensate comp time of 1/2 cyc
	goto	PwmInt		; instruc when timer int occured
PwmInt
;------------------------------------------------------------------
	bcf	INTCON,T0IF	; Clear TMR0 overflow flag

	movWf	W_TEMP		; Temporarily save W
	swapf	STATUS,W	; copy STATUS register...
	movWf	STATUS_TEMP	; ...to "stack"
;------------------------------------------------------------------
	btfsc	TONE_PIN	; Was this a Low cycle ?
	goto	PWMLow		; No ... 
;-------
;PWMHigh
	movFw	OUT_HIGH	; Yes... Load high time without affecting STATUS flags
	bsf	TONE_PIN
	nop			; Delay to equalize high/low TMR0 load cycles
	movWf	TMR0		; Load next edge interupt time

        goto	END_PWM
;-------
PWMLow	
	bcf	TONE_PIN
	movFw	OUT_LOW		; Load low time
	movWf	TMR0		; Load next edge interupt time
	nop

END_PWM
;-------------------------------------------------------------

	bsf	TMR0,7		;     64 uS ! ! !
	bsf	TMR0,6		;   OUT_HIGH  OUT_LOW		! ! !

	swapf	STATUS_TEMP,W	; restore ...
	movWf	STATUS		; ...STATUS register
	swapf	W_TEMP,F	; restore W register
	swapf	W_TEMP,W

	retfie			; return to main program
;******************************************************************
;			ISR   END
;******************************************************************

;--- power on routine ---------------------------------------------
START
				;    
	bsf	STATUS,RP0	; Bank 1
	call	H'3FF'		; Get the cal value
	movWf	OSCCAL		; Calibrate
	bcf	STATUS,RP0 	; Bank 0



; configuration of the PWM module
	clrf	TMR0		; reset timer
	bcf	TONE_PIN	; reset PWM-port before GPIO is changed

	movLw	B'00000111'
	movWf	CMCON		;  

				; configuration of the Pic
	bsf	STATUS,RP0	; register page 1
	movLw	C_TRIS_VAL	; configurate ...
	movWf	TRISIO		; ...GP port
	movLw	C_AN_SEL_VAL	;   / 
	movWf	ANSEL
	movLw	C_OPTION_VAL	; configurate ...
	movWf	OPTION_REG	; ...Pic
	bcf	STATUS,RP0	; register page 0

	movLw	C_ADCON0VAL	;  
	movWf	ADCON0

	movLw	B'00000001'	;  T1G
	movWf	T1CON		;  1:1
				;     OSC1
				;       
				;  
				;   TMR1

	clrf	TMR1L
	clrf	TMR1H
	clrf	GLOBAL_GONG_TMR

;-----------------------------------------------------------------------
	movLw	C_TONE_1_DECREMENT_STEP	;        
	movWf	COUNTER_TONE_1_DOWN

	movLw	C_TONE_2_DECREMENT_STEP
	movWf	COUNTER_TONE_2_DOWN

	movLw	C_TONE_3_DECREMENT_STEP
	movWf	COUNTER_TONE_3_DOWN

;-----------------------------------------------------------------------
	clrf	TONE_1_VOLUME		;       
	clrf	TONE_2_VOLUME
	clrf	TONE_2_VOLUME

;    ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! !
	movlw	H'1F'
	movWf	OUT_HIGH
	movWf	OUT_LOW

;***********************************************************************
	movLw	C_INTCONVAL		; configure interrupts and ...
	movWf	INTCON			; ...enable interrupts
;==== main idle =============================
MAIN
	clrwdt	; toggle watchdog

	tstf	GLOBAL_GONG_TMR
	skpnz
	goto	EMPTY_GLOBAL_GONG_TMR
;-------------------------------------------------------------------------
	btfss	PIR1,T1IF		;         TMR1
	goto	MAIN			; -> -> -> -> -> -> -> ->
;---------------------------------------
	bcf	PIR1,T1IF		;  
	decf	GLOBAL_GONG_TMR,F	;   .
;-------
	movFw	GLOBAL_GONG_TMR		;       
	xorLw	C_TONE_1_BEGIN_TIME


	movFw	GLOBAL_GONG_TMR		;       
	xorLw	C_TONE_2_BEGIN_TIME


	movFw	GLOBAL_GONG_TMR		;       
	xorLw	C_TONE_3_BEGIN_TIME


END_WAIT_TONES_START
;-----------------------------------------------------------------------
; DEC_VOLUME_1			;      (   '00')

	tstf	TONE_1_VOLUME
	skpnz
	goto	END_DEC_TONE_1

	decfsz	COUNTER_TONE_1_DOWN,F
	goto	END_DEC_TONE_1

	movLw	C_TONE_1_DECREMENT_STEP
	movWf	COUNTER_TONE_1_DOWN

	decf	TONE_1_VOLUME,F

END_DEC_TONE_1
;-----
;DEC_VOLUME_2			;      (   '00')

	tstf	TONE_2_VOLUME
	skpnz
	goto	END_DEC_TONE_2

	decfsz	COUNTER_TONE_2_DOWN,F
	goto	END_DEC_TONE_2

	movLw	C_TONE_2_DECREMENT_STEP
	movWf	COUNTER_TONE_2_DOWN

	decf	TONE_2_VOLUME,F

END_DEC_TONE_2
;-----
;DEC_VOLUME_3			;      (   '00')

	tstf	TONE_3_VOLUME
	skpnz
	goto	END_DEC_TONE_3

	decfsz	COUNTER_TONE_3_DOWN,F
	goto	END_DEC_TONE_3

	movLw	C_TONE_3_DECREMENT_STEP
	movWf	COUNTER_TONE_3_DOWN

	decf	TONE_3_VOLUME,F

END_DEC_TONE_3
;-----
;END_VOLUME_DECREMENT






















	goto	MAIN
;------------------------------------------------------------------------
EMPTY_GLOBAL_GONG_TMR

	movLw	C_TONE_1_DECREMENT_STEP
	movWf	COUNTER_TONE_1_DOWN

	movLw	C_TONE_2_DECREMENT_STEP
	movWf	COUNTER_TONE_2_DOWN

	movLw	C_TONE_3_DECREMENT_STEP
	movWf	COUNTER_TONE_3_DOWN
;-------
	btfss	DING_DANG_DONG_IN
	goto	MAIN

	movLw	C_GLOBAL_GONG_TIME
	movWf	GLOBAL_GONG_TMR



	nop
	nop


	goto	MAIN		; ...then waiting

;------------------------------------------------------------------

	ORG H'3FF'

	retlw	H'64'		;   


	END
